# Author: Addison Sears-Collins
# Date: September 2, 2021
# Description: Launch a basic mobile robot using the ROS 2 Navigation Stack
# https://automaticaddison.com

import os
from pathlib import Path
from ament_index_python import get_package_share_directory
from launch import LaunchDescription
from launch.actions import DeclareLaunchArgument, IncludeLaunchDescription, SetEnvironmentVariable
from launch.launch_description_sources import PythonLaunchDescriptionSource
from launch.substitutions import Command, LaunchConfiguration, PythonExpression
from launch_ros.substitutions import FindPackageShare
from launch_ros.actions import Node, SetRemap
from launch.actions import GroupAction

def generate_launch_description():

  # Set the path to different files and folders.
  pkg_share = get_package_share_directory('dcaiti_navigation')
  nav2_params_path = os.path.join(pkg_share, 'params', 'nav2_params.yaml')
  behavior_tree_xml_path = os.path.join(get_package_share_directory('nav2_bt_navigator'), 'behavior_trees', 'navigate_w_replanning_and_recovery.xml')
  
  # Launch configuration variables specific to simulation
  autostart = LaunchConfiguration('autostart')
  default_bt_xml_filename = LaunchConfiguration('default_bt_xml_filename')
  map_yaml_file = LaunchConfiguration('map')
  namespace = LaunchConfiguration('namespace')
  params_file = LaunchConfiguration('params_file')
  slam = LaunchConfiguration('slam')
  use_namespace = LaunchConfiguration('use_namespace')
  use_sim_time = LaunchConfiguration('use_sim_time')
  
  # Declare the launch arguments  
  declare_namespace_cmd = DeclareLaunchArgument(
    name='namespace',
    default_value='',
    description='Top-level namespace')

  declare_use_namespace_cmd = DeclareLaunchArgument(
    name='use_namespace',
    default_value='False',
    description='Whether to apply a namespace to the navigation stack')
        
  declare_autostart_cmd = DeclareLaunchArgument(
    name='autostart', 
    default_value='true',
    description='Automatically startup the nav2 stack')

  declare_bt_xml_cmd = DeclareLaunchArgument(
    name='default_bt_xml_filename',
    default_value=behavior_tree_xml_path,
    description='Full path to the behavior tree xml file to use')
    
  declare_params_file_cmd = DeclareLaunchArgument(
    name='params_file',
    default_value=nav2_params_path,
    description='Full path to the ROS2 parameters file to use for all launched nodes')
    
  declare_use_sim_time_cmd = DeclareLaunchArgument(
    name='use_sim_time',
    default_value='False',
    description='Use simulation (Gazebo) clock if true')

  slam_params = os.path.join(
        get_package_share_directory('dcaiti_navigation'), # <-- Replace with your package name
        'params',
        'nav2_params.yaml'
        )
  

  start_ros2_navigation_cmd = GroupAction(
    actions=[
        SetRemap(src='/cmd_vel',dst='/cmd_vel_nav'),
        IncludeLaunchDescription(
    PythonLaunchDescriptionSource([os.path.join(
    get_package_share_directory('nav2_bringup'),'launch','navigation_launch.py')]),
    launch_arguments={
                'use_sim_time': 'false',
                '-r': '/cmd_vel:=/cmd_vel_nav'
    }.items()
    )
    ]
)
  

  
  slam_toolbox = IncludeLaunchDescription(
            PythonLaunchDescriptionSource([os.path.join(
                get_package_share_directory('slam_toolbox'),'launch','online_async_launch.py'
            )]), 
            launch_arguments={
                'use_sim_time': 'false', 
                'slam_params_file': slam_params}.items()
  )


  node_fix_lidar = Node(
        package="dcaitirobot",
        executable="fixlidar",
        name="fixlidar",
    )

  tfs1 = Node(package = "tf2_ros", 
                      executable = "static_transform_publisher",
                      arguments = ["0", "0", "0" ,"0", "0", "0", "base_link", "velodyne"])

  tfs2 = Node(package = "tf2_ros", 
                      executable = "static_transform_publisher",
                      arguments = ["0", "0", "0" ,"0", "0", "0", "base_link", "base_footprint"])



  exploration = IncludeLaunchDescription(
            PythonLaunchDescriptionSource([os.path.join(
                get_package_share_directory('explore_lite'),'launch','explore.launch.py'
            )]), 
            launch_arguments={
                'use_sim_time': 'false'}.items()
  )

  # Create the launch description and populate
  ld = LaunchDescription([
      declare_namespace_cmd,
      declare_use_namespace_cmd,
      declare_autostart_cmd,
      declare_bt_xml_cmd,
      declare_params_file_cmd,
      declare_use_sim_time_cmd,
      start_ros2_navigation_cmd,
      slam_toolbox,
      node_fix_lidar,
      tfs1,
      tfs2,
      exploration,
  ])

  return ld
